#!/usr/local/bin/python
#
# Copyright (c) 2004 Niels Provos <provos@citi.umich.edu>
# All rights reserved.
#
import string
import sys
import os
import re

def addresstonumber(address):
    '''Converts an IPv4 in string to a 32-bit number.'''
    tuples = string.split(address, ".")
    sum = 0
    for tuple in tuples:
        sum = (sum << 8) + int(tuple)
    return sum & 0xffffffffL

def numbertoaddress(number):
    '''Converts a 32-bit number to an IPv4 string'''
    address = ""
    count = 0
    while count < 4:
        count += 1
        addr = str(number % 256)
        number = number >> 8
        if address:
            addr = addr+"."
        address = addr + address
    return address

def addresstominmax(address):
    '''Returns the start and the end of a CIDR network.'''
    rawlist = string.split(address, "/")
    if len(rawlist) == 1:
        return [address, address]

    maxmask = (1 << (32 - int(rawlist[1]))) - 1
    minmask = (0xffffffffl << (32 - int(rawlist[1])))
    minaddr = numbertoaddress(addresstonumber(rawlist[0]) & minmask)
    maxaddr = numbertoaddress(addresstonumber(minaddr) | maxmask)

    return [minaddr, maxaddr]

def addresscombine(addrlist):
    min = (256*256*256*256 - 1) & 0xffffffffl
    max = 0
    for address in addrlist:
        (minaddr, maxaddr) = addresstominmax(address)
        minnumber = addresstonumber(minaddr)
        maxnumber = addresstonumber(maxaddr)
        if minnumber < min:
            min = minnumber
        if maxnumber > max:
            max = maxnumber

    list = []
    where = min
    while where < max:
        currentmax = max
        # Check which networks should be excluded
        for (exstart, exend) in excluded:
            if where >= exstart and where <= exend:
                where = exend + 1
                continue
            if currentmax >= exstart and currentmax <= exend:
                currentmax = exstart - 1
                if currentmax < where:
                    currentmax = where
                    continue
            if where < exstart and currentmax > exend:
                currentmax = exstart - 1
            
        mask = 32
        while mask >= 0:
            nrmask = (0x1l << (32 - mask)) - 1
            minmask = (0xffffffffl << (32 - mask)) & 0xffffffffl
            if (where | nrmask) > currentmax:
                break
            if (where & minmask) != where:
                break
            mask -= 1
        mask += 1

        minmask = (0xffffffffl << (32 - mask)) & 0xffffffffl
        list.append(numbertoaddress(where & minmask)+"/"+str(mask))

        # Find the next boundary
        nrmask = (0x1l << (32 - mask)) - 1
        where = where | nrmask
        where += 1

    return list
        

def make_tree(routers, level, children, parents):
    if showtree:
        print routers['self']

    myparents=[]
    for router in parents:
        myparents.append(router)
    myparents.append(routers['self'])
                   
    list = []
    nets = []
    for router in routers.keys():
        if not ipre.match(router):
            continue
        try:
            index = parents.index(router)
        except:
            index = None
        if index:
            continue
        list.append(router)
        nets.append(router)

    while len(list):
        router = list.pop(0)
        i = 0
        if showtree:
            while i < level - 1:
                if children & (1 << i):
                    print " |",
                else:
                    print "  ",
                i += 1
            if level > 0:
                print " -",
        if len(list):
            havemore = children | (1 << level)
        else:
            havemore = children
        make_tree(routers[router], level + 1, havemore, myparents)

    if not len(nets):
        addr = routers['self']
        net = string.join(string.split(addr, ".")[0:3], ".")+".0/24"
        routers['link'] = net
        routers['net'] = [ net ]
    else:
        list = []
        for router in nets:
            list.extend(routers[router]['net'])
        net = addresscombine(list)
        routers['net'] = net

    if debug:
        print >>sys.stderr, "%s -> %s" % (routers['self'], routers['net'])

    if showroute:
        if level == 0:
            print 'route entry %s' % routers['self']
        if routers.has_key('link'):
            print 'route %s link %s' % (routers['self'], routers['link'])
        else:
            print 'route %s link %s/32' % (routers['self'], routers['self'])
        for address in routers.keys():
            if not ipre.match(address):
                continue
            router = routers[address]
            if not router.has_key('net'):
                continue
            for net in router['net']:
                print 'route %s add net %s %s' % \
                      (routers['self'], net, router['self'])

# Main
debug = 0
showtree = 0
showroute = 1

# Addresses to exclude
excluded = []
for (addrstart, addrend) in [addresstominmax('127.0.0.0/8')]:
    excluded.append([addresstonumber(addrstart), addresstonumber(addrend)])


ipre = re.compile("^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$")

routers = {}

count=0
for line in sys.stdin:
    count+=1
    
    m = re.match("^(.*): (.*)$", line)
    if not m:
        print >>sys.stderr, "Cannot parse line %d: %s\n" % (count, line)

    dst = m.group(1)
    list = m.group(2)

    routes = string.split(list, " ")
    parent = routers
    parent['self'] = '0.0.0.0'
    oldrouter = None
    for router in routes:
        if not re.match("^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$", router):
            break
        if not parent.has_key(router):
            parent[router] = {}
        parent = parent[router]
        parent['self'] = router
        if oldrouter:
            parent['parent'] = oldrouter
        oldrouter = router;

make_tree(routers, 0, 0, [])
