/*
 * Copyright 2003 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _NTTLSCAN_H_
#define _NTTLSCAN_H_

#define NTTL_BASE_SPORT		0xaffe
#define NTTL_BASE_DPORT		0xdead

#define NTTL_MAXINFLIGHT	100

#define TCP_CONTENT	"Hello.  This is a probe packet!"

#define HOP_UNREACH	0x0001
#define HOP_ENDHOST	0x0002
#define HOP_STORED	0x0004
#define HOP_TIMEOUT	0x0008

struct hop {
	struct probe *parent;
	struct event ev;

	struct addr addr;	/* gateway or endhost */

	struct timeval sent;
	struct timeval received;

	u_short flags;

	u_char ttl;
	u_char count;

	u_char unreach;		/* code for unreachability */
};

#define DECODE_EXCEED		0
#define DECODE_UNREACH		1
#define DECODE_TCP_TCP		2

#define PROBE_MAXHOPS		64
#define PROBE_MAXRETRIES	3

#define PROBE_INITIALDELAY	5
#define PROBE_TIMEOUT		500

struct probe {
	SPLAY_ENTRY(probe) node;

	struct addr dst;
	struct addr src;

	struct hop *hops[PROBE_MAXHOPS];
	int nhops;

	int active;		/* Currently active hop probes */
};

struct udp_payload {
	uint8_t ttl;
	uint8_t count;
};

void make_random_ip(struct addr *, struct addr *);
int make_probe(struct addr *);

int tcp_probe(struct addr *, struct addr *, int, int);
void decode_tcp_probe(struct addr *, struct ip_hdr *, int, int);
void decode_tcp_connect(struct addr *addr, struct ip_hdr *ip);

int udp_probe(struct addr *, struct addr *, int, int);
void decode_udp_probe(struct addr *, struct ip_hdr *, int, int);

void store_result(struct probe *, struct addr *, u_int, u_int, int, int);
void schedule_probe(struct hop *, int);
void allocate_hops(struct probe *, int);
void nttl_exit(int);

int addr_inrange(struct addr *, struct addr *);

void tcp_setup(int);

#endif
